@@ -15,6 +15,7 @@
 
 #include "postgres.h"
 
+#include "distributed/metadata_utility.h"
 #include "utils/rel.h"
 #include "nodes/parsenodes.h"
 #include "tcop/dest.h"
@@ -23,6 +24,7 @@
 
 
 extern bool AddAllLocalTablesToMetadata;
+extern bool EnableSchemaBasedSharding;
 
 /* controlled via GUC, should be accessed via EnableLocalReferenceForeignKeys() */
 extern bool EnableLocalReferenceForeignKeys;
@@ -116,7 +118,7 @@ typedef enum ExtractForeignKeyConstraintsMode
 	/* exclude the self-referencing foreign keys */
 	EXCLUDE_SELF_REFERENCES = 1 << 2,
 
-	/* any combination of the 4 flags below is supported */
+	/* any combination of the 5 flags below is supported */
 	/* include foreign keys when the other table is a distributed table*/
 	INCLUDE_DISTRIBUTED_TABLES = 1 << 3,
 
@@ -129,9 +131,13 @@ typedef enum ExtractForeignKeyConstraintsMode
 	/* include foreign keys when the other table is a Postgres local table*/
 	INCLUDE_LOCAL_TABLES = 1 << 6,
 
+	/* include foreign keys when the other table is a single shard table*/
+	INCLUDE_SINGLE_SHARD_TABLES = 1 << 7,
+
 	/* include foreign keys regardless of the other table's type */
 	INCLUDE_ALL_TABLE_TYPES = INCLUDE_DISTRIBUTED_TABLES | INCLUDE_REFERENCE_TABLES |
-							  INCLUDE_CITUS_LOCAL_TABLES | INCLUDE_LOCAL_TABLES
+							  INCLUDE_CITUS_LOCAL_TABLES | INCLUDE_LOCAL_TABLES |
+							  INCLUDE_SINGLE_SHARD_TABLES
 } ExtractForeignKeyConstraintMode;
 
 
@@ -153,6 +159,19 @@ typedef enum SearchForeignKeyColumnFlags
 	/* callers can also pass union of above flags */
 } SearchForeignKeyColumnFlags;
 
+
+typedef enum TenantOperation
+{
+	TENANT_UNDISTRIBUTE_TABLE = 0,
+	TENANT_ALTER_TABLE,
+	TENANT_COLOCATE_WITH,
+	TENANT_UPDATE_COLOCATION,
+	TENANT_SET_SCHEMA,
+} TenantOperation;
+
+#define TOTAL_TENANT_OPERATION 5
+extern const char *TenantOperationNames[TOTAL_TENANT_OPERATION];
+
 /* begin.c - forward declarations */
 extern void SaveBeginCommandProperties(TransactionStmt *transactionStmt);
 
@@ -201,6 +220,17 @@ extern List * AlterDatabaseOwnerObjectAddress(Node *node, bool missing_ok, bool
 											  isPostprocess);
 extern List * DatabaseOwnerDDLCommands(const ObjectAddress *address);
 
+extern List * PreprocessGrantOnDatabaseStmt(Node *node, const char *queryString,
+											ProcessUtilityContext processUtilityContext);
+
+extern List * PreprocessAlterDatabaseStmt(Node *node, const char *queryString,
+										  ProcessUtilityContext processUtilityContext);
+
+extern List * PreprocessAlterDatabaseRefreshCollStmt(Node *node, const char *queryString,
+													 ProcessUtilityContext
+													 processUtilityContext);
+
+
 /* domain.c - forward declarations */
 extern List * CreateDomainStmtObjectAddress(Node *node, bool missing_ok, bool
 											isPostprocess);
@@ -216,6 +246,7 @@ extern List * RenameDomainStmtObjectAddress(Node *node, bool missing_ok, bool
 extern CreateDomainStmt * RecreateDomainStmt(Oid domainOid);
 extern Oid get_constraint_typid(Oid conoid);
 
+
 /* extension.c - forward declarations */
 extern bool IsDropCitusExtensionStmt(Node *parsetree);
 extern List * GetDependentFDWsToExtension(Oid extensionId);
@@ -269,9 +300,14 @@ extern bool ColumnAppearsInForeignKeyToReferenceTable(char *columnName, Oid
 													  relationId);
 extern List * GetReferencingForeignConstaintCommands(Oid relationOid);
 extern List * GetForeignConstraintToReferenceTablesCommands(Oid relationId);
+extern List * GetForeignConstraintFromOtherReferenceTablesCommands(Oid relationId);
 extern List * GetForeignConstraintToDistributedTablesCommands(Oid relationId);
 extern List * GetForeignConstraintFromDistributedTablesCommands(Oid relationId);
 extern List * GetForeignConstraintCommandsInternal(Oid relationId, int flags);
+extern Oid DropFKeysAndUndistributeTable(Oid relationId);
+extern void DropFKeysRelationInvolvedWithTableType(Oid relationId, int tableTypeFlag);
+extern List * GetFKeyCreationCommandsRelationInvolvedWithTableType(Oid relationId,
+																   int tableTypeFlag);
 extern bool AnyForeignKeyDependsOnIndex(Oid indexId);
 extern bool HasForeignKeyWithLocalTable(Oid relationId);
 extern bool HasForeignKeyToReferenceTable(Oid relationOid);
@@ -409,6 +445,24 @@ extern void DropPolicyEventExtendNames(DropStmt *stmt, const char *schemaName, u
 
 extern void AddRangeTableEntryToQueryCompat(ParseState *parseState, Relation relation);
 
+/* publication.c - forward declarations */
+extern List * PostProcessCreatePublicationStmt(Node *node, const char *queryString);
+extern List * CreatePublicationDDLCommandsIdempotent(const ObjectAddress *address);
+extern char * CreatePublicationDDLCommand(Oid publicationId);
+extern List * PreprocessAlterPublicationStmt(Node *stmt, const char *queryString,
+											 ProcessUtilityContext processUtilityCtx);
+extern List * GetAlterPublicationDDLCommandsForTable(Oid relationId, bool isAdd);
+extern char * GetAlterPublicationTableDDLCommand(Oid publicationId, Oid relationId,
+												 bool isAdd);
+extern List * AlterPublicationOwnerStmtObjectAddress(Node *node, bool missingOk,
+													 bool isPostProcess);
+extern List * AlterPublicationStmtObjectAddress(Node *node, bool missingOk,
+												bool isPostProcess);
+extern List * CreatePublicationStmtObjectAddress(Node *node, bool missingOk,
+												 bool isPostProcess);
+extern List * RenamePublicationStmtObjectAddress(Node *node, bool missingOk,
+												 bool isPostProcess);
+
 /* rename.c - forward declarations*/
 extern List * PreprocessRenameStmt(Node *renameStmt, const char *renameCommand,
 								   ProcessUtilityContext processUtilityContext);
@@ -440,8 +494,7 @@ extern void UnmarkRolesDistributed(List *roles);
 extern List * FilterDistributedRoles(List *roles);
 
 /* schema.c - forward declarations */
-extern List * PreprocessCreateSchemaStmt(Node *node, const char *queryString,
-										 ProcessUtilityContext processUtilityContext);
+extern List * PostprocessCreateSchemaStmt(Node *node, const char *queryString);
 extern List * PreprocessDropSchemaStmt(Node *dropSchemaStatement,
 									   const char *queryString,
 									   ProcessUtilityContext processUtilityContext);
@@ -451,6 +504,8 @@ extern List * PreprocessGrantOnSchemaStmt(Node *node, const char *queryString,
 										  ProcessUtilityContext processUtilityContext);
 extern List * CreateSchemaStmtObjectAddress(Node *node, bool missing_ok, bool
 											isPostprocess);
+extern List * AlterSchemaOwnerStmtObjectAddress(Node *node, bool missing_ok,
+												bool isPostprocess);
 extern List * AlterSchemaRenameStmtObjectAddress(Node *node, bool missing_ok, bool
 												 isPostprocess);
 
@@ -540,6 +595,8 @@ extern bool ShouldEnableLocalReferenceForeignKeys(void);
 extern List * PreprocessAlterTableStmtAttachPartition(AlterTableStmt *alterTableStatement,
 													  const char *queryString);
 extern List * PostprocessAlterTableSchemaStmt(Node *node, const char *queryString);
+extern void PrepareAlterTableStmtForConstraint(AlterTableStmt *alterTableStatement,
+											   Oid relationId, Constraint *constraint);
 extern List * PreprocessAlterTableStmt(Node *node, const char *alterTableCommand,
 									   ProcessUtilityContext processUtilityContext);
 extern List * PreprocessAlterTableMoveAllStmt(Node *node, const char *queryString,
@@ -557,6 +614,8 @@ extern void ErrorUnsupportedAlterTableAddColumn(Oid relationId, AlterTableCmd *c
 extern void ErrorIfUnsupportedConstraint(Relation relation, char distributionMethod,
 										 char referencingReplicationModel,
 										 Var *distributionColumn, uint32 colocationId);
+extern List * InterShardDDLTaskList(Oid leftRelationId, Oid rightRelationId,
+									const char *commandString);
 extern List * AlterTableSchemaStmtObjectAddress(Node *stmt,
 												bool missing_ok, bool isPostprocess);
 extern List * MakeNameListFromRangeVar(const RangeVar *rel);
@@ -566,6 +625,10 @@ extern bool ConstrTypeCitusCanDefaultName(ConstrType constrType);
 extern char * GetAlterColumnWithNextvalDefaultCmd(Oid sequenceOid, Oid relationId,
 												  char *colname, bool missingTableOk);
 
+extern void ErrorIfTableHasIdentityColumn(Oid relationId);
+extern void ConvertNewTableIfNecessary(Node *createStmt);
+extern void ConvertToTenantTableIfNecessary(AlterObjectSchemaStmt *alterObjectSchemaStmt);
+
 /* text_search.c - forward declarations */
 extern List * GetCreateTextSearchConfigStatements(const ObjectAddress *address);
 extern List * GetCreateTextSearchDictionaryStatements(const ObjectAddress *address);
@@ -657,7 +720,6 @@ extern List * PreprocessDropViewStmt(Node *node, const char *queryString,
 extern List * DropViewStmtObjectAddress(Node *node, bool missing_ok, bool isPostprocess);
 extern char * CreateViewDDLCommand(Oid viewOid);
 extern List * GetViewCreationCommandsOfTable(Oid relationId);
-extern List * GetViewCreationTableDDLCommandsOfTable(Oid relationId);
 extern char * AlterViewOwnerCommand(Oid viewOid);
 extern char * DeparseViewStmt(Node *node);
 extern char * DeparseDropViewStmt(Node *node);
@@ -750,6 +812,7 @@ extern void ExecuteForeignKeyCreateCommandList(List *ddlCommandList,
 /* create_citus_local_table.c */
 extern void CreateCitusLocalTable(Oid relationId, bool cascadeViaForeignKeys,
 								  bool autoConverted);
+extern bool ShouldAddNewTableToMetadata(Oid relationId);
 extern List * GetExplicitIndexOidList(Oid relationId);
 
 extern bool ShouldPropagateSetCommand(VariableSetStmt *setStmt);
@@ -760,4 +823,14 @@ extern void CreateCitusLocalTablePartitionOf(CreateStmt *createStatement,
 extern void UpdateAutoConvertedForConnectedRelations(List *relationId, bool
 													 autoConverted);
 
+/* schema_based_sharding.c */
+extern bool ShouldUseSchemaBasedSharding(char *schemaName);
+extern bool ShouldCreateTenantSchemaTable(Oid relationId);
+extern void EnsureTenantTable(Oid relationId, char *operationName);
+extern void ErrorIfIllegalPartitioningInTenantSchema(Oid parentRelationId,
+													 Oid partitionRelationId);
+extern void CreateTenantSchemaTable(Oid relationId);
+extern void ErrorIfTenantTable(Oid relationId, const char *operationName);
+extern uint32 CreateTenantSchemaColocationId(void);
+
 #endif /*CITUS_COMMANDS_H */
