@@ -42,10 +42,31 @@
 /* application name used for connections made by run_command_on_* */
 #define CITUS_RUN_COMMAND_APPLICATION_NAME_PREFIX "citus_run_command gpid="
 
+/*
+ * application name prefix for move/split replication connections.
+ *
+ * This application_name is set to the subscription name by logical replication
+ * workers, so there is no GPID.
+ */
+#define CITUS_SHARD_TRANSFER_APPLICATION_NAME_PREFIX "citus_shard_"
+
 /* deal with waiteventset errors */
 #define WAIT_EVENT_SET_INDEX_NOT_INITIALIZED -1
 #define WAIT_EVENT_SET_INDEX_FAILED -2
 
+/*
+ * UINT32_MAX is reserved in pg_dist_node, so we can use it safely.
+ */
+#define LOCAL_NODE_ID UINT32_MAX
+
+/*
+ * If you want to connect to the current node use `LocalHostName`, which is a GUC, instead
+ * of the hardcoded loopback hostname. Only if you really need the loopback hostname use
+ * this define.
+ */
+#define LOCAL_HOST_NAME "localhost"
+
+
 /* forward declare, to avoid forcing large headers on everyone */
 struct pg_conn; /* target of the PGconn typedef */
 struct MemoryContextData;
@@ -113,7 +134,7 @@ enum MultiConnectionMode
 
 
 /*
- * This state is used for keeping track of the initilization
+ * This state is used for keeping track of the initialization
  * of the underlying pg_conn struct.
  */
 typedef enum MultiConnectionState
@@ -128,7 +149,7 @@ typedef enum MultiConnectionState
 
 
 /*
- * This state is used for keeping track of the initilization
+ * This state is used for keeping track of the initialization
  * of MultiConnection struct, not specifically the underlying
  * pg_conn. The state is useful to determine the action during
  * clean-up of connections.
@@ -173,6 +194,9 @@ typedef struct MultiConnection
 	/* is the connection currently in use, and shouldn't be used by anything else */
 	bool claimedExclusively;
 
+	/* is the replication origin session has already been setup for this connection. */
+	bool isReplicationOriginSessionSetup;
+
 	/*
 	 * Should be used to access/modify metadata. See REQUIRE_METADATA_CONNECTION for
 	 * the details.
@@ -183,7 +207,7 @@ typedef struct MultiConnection
 	instr_time connectionEstablishmentStart;
 	instr_time connectionEstablishmentEnd;
 
-	/* membership in list of list of connections in ConnectionHashEntry */
+	/* membership in list of connections in ConnectionHashEntry */
 	dlist_node connectionNode;
 
 	/* information about the associated remote transaction */
@@ -205,7 +229,7 @@ typedef struct MultiConnection
 	/* replication option */
 	bool requiresReplication;
 
-	MultiConnectionStructInitializationState initilizationState;
+	MultiConnectionStructInitializationState initializationState;
 } MultiConnection;
 
 
@@ -261,6 +285,7 @@ extern int MaxCachedConnectionLifetime;
 /* parameters used for outbound connections */
 extern char *NodeConninfo;
 extern char *LocalHostName;
+extern bool checkAtBootPassed;
 
 /* the hash tables are externally accessiable */
 extern HTAB *ConnectionHash;
@@ -312,6 +337,7 @@ extern void ShutdownConnection(MultiConnection *connection);
 /* dealing with a connection */
 extern void FinishConnectionListEstablishment(List *multiConnectionList);
 extern void FinishConnectionEstablishment(MultiConnection *connection);
+extern void ForceConnectionCloseAtTransactionEnd(MultiConnection *connection);
 extern void ClaimConnectionExclusively(MultiConnection *connection);
 extern void UnclaimConnection(MultiConnection *connection);
 extern void MarkConnectionConnected(MultiConnection *connection);
@@ -327,7 +353,4 @@ extern bool CitusModifyWaitEvent(WaitEventSet *set, int pos, uint32 events,
 extern double MillisecondsPassedSince(instr_time moment);
 extern long MillisecondsToTimeout(instr_time start, long msAfterStart);
 
-#if PG_VERSION_NUM < 140000
-extern void WarmUpConnParamsHash(void);
-#endif
 #endif /* CONNECTION_MANAGMENT_H */
