@@ -105,9 +105,22 @@ extern char * DeparseCreateSchemaStmt(Node *node);
 extern char * DeparseDropSchemaStmt(Node *node);
 extern char * DeparseGrantOnSchemaStmt(Node *stmt);
 extern char * DeparseAlterSchemaRenameStmt(Node *stmt);
+extern char * DeparseAlterSchemaOwnerStmt(Node *node);
 
 extern void AppendGrantPrivileges(StringInfo buf, GrantStmt *stmt);
 extern void AppendGrantGrantees(StringInfo buf, GrantStmt *stmt);
+extern void AppendWithGrantOption(StringInfo buf, GrantStmt *stmt);
+extern void AppendGrantOptionFor(StringInfo buf, GrantStmt *stmt);
+extern void AppendGrantRestrictAndCascadeForRoleSpec(StringInfo buf, DropBehavior
+													 behavior, bool isGrant);
+extern void AppendGrantRestrictAndCascade(StringInfo buf, GrantStmt *stmt);
+extern void AppendGrantedByInGrantForRoleSpec(StringInfo buf, RoleSpec *grantor, bool
+											  isGrant);
+extern void AppendGrantedByInGrant(StringInfo buf, GrantStmt *stmt);
+
+extern void AppendGrantSharedPrefix(StringInfo buf, GrantStmt *stmt);
+extern void AppendGrantSharedSuffix(StringInfo buf, GrantStmt *stmt);
+
 
 /* forward declarations for deparse_statistics_stmts.c */
 extern char * DeparseCreateStatisticsStmt(Node *node);
@@ -209,6 +222,26 @@ extern char * DeparseAlterExtensionStmt(Node *stmt);
 
 /* forward declarations for deparse_database_stmts.c */
 extern char * DeparseAlterDatabaseOwnerStmt(Node *node);
+extern char * DeparseGrantOnDatabaseStmt(Node *node);
+extern char * DeparseAlterDatabaseStmt(Node *node);
+extern char * DeparseAlterDatabaseRefreshCollStmt(Node *node);
+
+/* forward declaration for deparse_publication_stmts.c */
+extern char * DeparseCreatePublicationStmt(Node *stmt);
+extern char * DeparseCreatePublicationStmtExtended(Node *node,
+												   bool whereClauseNeedsTransform,
+												   bool includeLocalTables);
+extern char * DeparseAlterPublicationStmt(Node *stmt);
+extern char * DeparseAlterPublicationStmtExtended(Node *stmt,
+												  bool whereClauseNeedsTransform,
+												  bool includeLocalTables);
+extern char * DeparseAlterPublicationOwnerStmt(Node *stmt);
+extern char * DeparseAlterPublicationSchemaStmt(Node *node);
+extern char * DeparseDropPublicationStmt(Node *stmt);
+extern char * DeparseRenamePublicationStmt(Node *node);
+
+extern void QualifyCreatePublicationStmt(Node *node);
+extern void QualifyAlterPublicationStmt(Node *node);
 
 /* forward declatations for deparse_text_search_stmts.c */
 extern void QualifyAlterTextSearchConfigurationOwnerStmt(Node *node);
