@@ -18,9 +18,31 @@
 #include "distributed/metadata_cache.h"
 #include "nodes/pg_list.h"
 
+/* managed via guc.c */
+typedef enum
+{
+	METADATA_SYNC_TRANSACTIONAL = 0,
+	METADATA_SYNC_NON_TRANSACTIONAL = 1
+} MetadataSyncTransactionMode;
+
 /* config variables */
 extern int MetadataSyncInterval;
 extern int MetadataSyncRetryInterval;
+extern int MetadataSyncTransMode;
+
+/*
+ * MetadataSyncContext is used throughout metadata sync.
+ */
+typedef struct MetadataSyncContext
+{
+	List *activatedWorkerNodeList; /* activated worker nodes */
+	List *activatedWorkerBareConnections; /* bare connections to activated nodes */
+	MemoryContext context; /* memory context for all allocations */
+	MetadataSyncTransactionMode transactionMode; /* transaction mode for the sync */
+	bool collectCommands; /* if we collect commands instead of sending and resetting */
+	List *collectedCommands; /* collected commands. (NIL if collectCommands == false) */
+	bool nodesAddedInSameTransaction; /* if the nodes are added just before activation */
+} MetadataSyncContext;
 
 typedef enum
 {
@@ -52,7 +74,6 @@ extern void citus_internal_add_placement_metadata_internal(int64 shardId,
 														   int64 shardLength,
 														   int32 groupId,
 														   int64 placementId);
-extern void SyncNodeMetadataToNode(const char *nodeNameString, int32 nodePort);
 extern void SyncCitusTableMetadata(Oid relationId);
 extern void EnsureSequentialModeMetadataOperations(void);
 extern bool ClusterHasKnownMetadataWorkers(void);
@@ -60,10 +81,10 @@ extern char * LocalGroupIdUpdateCommand(int32 groupId);
 extern bool ShouldSyncUserCommandForObject(ObjectAddress objectAddress);
 extern bool ShouldSyncTableMetadata(Oid relationId);
 extern bool ShouldSyncTableMetadataViaCatalog(Oid relationId);
+extern Oid FetchRelationIdFromPgPartitionHeapTuple(HeapTuple heapTuple,
+												   TupleDesc tupleDesc);
 extern bool ShouldSyncSequenceMetadata(Oid relationId);
 extern List * NodeMetadataCreateCommands(void);
-extern List * DistributedObjectMetadataSyncCommandList(void);
-extern List * ColocationGroupCreateCommandList(void);
 extern List * CitusTableMetadataCreateCommandList(Oid relationId);
 extern List * NodeMetadataDropCommands(void);
 extern char * MarkObjectsDistributedCreateCommand(List *addresses,
@@ -76,6 +97,7 @@ extern char * DistributionDeleteCommand(const char *schemaName,
 extern char * DistributionDeleteMetadataCommand(Oid relationId);
 extern char * TableOwnerResetCommand(Oid distributedRelationId);
 extern char * NodeListInsertCommand(List *workerNodeList);
+char * NodeListIdempotentInsertCommand(List *workerNodeList);
 extern List * ShardListInsertCommand(List *shardIntervalList);
 extern List * ShardDeleteCommandList(ShardInterval *shardInterval);
 extern char * NodeDeleteCommand(uint32 nodeId);
@@ -97,15 +119,16 @@ extern List * InterTableRelationshipOfRelationCommandList(Oid relationId);
 extern List * DetachPartitionCommandList(void);
 extern void SyncNodeMetadataToNodes(void);
 extern BackgroundWorkerHandle * SpawnSyncNodeMetadataToNodes(Oid database, Oid owner);
-extern void SyncNodeMetadataToNodesMain(Datum main_arg);
+extern PGDLLEXPORT void SyncNodeMetadataToNodesMain(Datum main_arg);
 extern void SignalMetadataSyncDaemon(Oid database, int sig);
 extern bool ShouldInitiateMetadataSync(bool *lockFailure);
 extern List * SequenceDependencyCommandList(Oid relationId);
+extern List * IdentitySequenceDependencyCommandList(Oid targetRelationId);
 
 extern List * DDLCommandsForSequence(Oid sequenceOid, char *ownerName);
 extern List * GetSequencesFromAttrDef(Oid attrdefOid);
 extern void GetDependentSequencesWithRelation(Oid relationId, List **seqInfoList,
-											  AttrNumber attnum);
+											  AttrNumber attnum, char depType);
 extern List * GetDependentFunctionsWithRelation(Oid relationId);
 extern Oid GetAttributeTypeOid(Oid relationId, AttrNumber attnum);
 extern void SetLocalEnableMetadataSync(bool state);
@@ -114,6 +137,44 @@ extern void SyncNewColocationGroupToNodes(uint32 colocationId, int shardCount,
 										  Oid distributionColumType,
 										  Oid distributionColumnCollation);
 extern void SyncDeleteColocationGroupToNodes(uint32 colocationId);
+extern char * TenantSchemaInsertCommand(Oid schemaId, uint32 colocationId);
+extern char * TenantSchemaDeleteCommand(char *schemaName);
+extern char * UpdateNoneDistTableMetadataCommand(Oid relationId, char replicationModel,
+												 uint32 colocationId, bool autoConverted);
+extern char * AddPlacementMetadataCommand(uint64 shardId, uint64 placementId,
+										  uint64 shardLength, int32 groupId);
+extern char * DeletePlacementMetadataCommand(uint64 placementId);
+
+extern MetadataSyncContext * CreateMetadataSyncContext(List *nodeList,
+													   bool collectCommands,
+													   bool nodesAddedInSameTransaction);
+extern void EstablishAndSetMetadataSyncBareConnections(MetadataSyncContext *context);
+extern void SetMetadataSyncNodesFromNodeList(MetadataSyncContext *context,
+											 List *nodeList);
+extern void ResetMetadataSyncMemoryContext(MetadataSyncContext *context);
+extern bool MetadataSyncCollectsCommands(MetadataSyncContext *context);
+extern void SendOrCollectCommandListToActivatedNodes(MetadataSyncContext *context,
+													 List *commands);
+extern void SendOrCollectCommandListToMetadataNodes(MetadataSyncContext *context,
+													List *commands);
+extern void SendOrCollectCommandListToSingleNode(MetadataSyncContext *context,
+												 List *commands, int nodeIdx);
+
+extern void ActivateNodeList(MetadataSyncContext *context);
+
+extern char * WorkerDropAllShellTablesCommand(bool singleTransaction);
+extern char * WorkerDropSequenceDependencyCommand(Oid relationId);
+
+extern void SyncDistributedObjects(MetadataSyncContext *context);
+extern void SendNodeWideObjectsSyncCommands(MetadataSyncContext *context);
+extern void SendShellTableDeletionCommands(MetadataSyncContext *context);
+extern void SendMetadataDeletionCommands(MetadataSyncContext *context);
+extern void SendColocationMetadataCommands(MetadataSyncContext *context);
+extern void SendTenantSchemaMetadataCommands(MetadataSyncContext *context);
+extern void SendDependencyCreationCommands(MetadataSyncContext *context);
+extern void SendDistTableMetadataCommands(MetadataSyncContext *context);
+extern void SendDistObjectCommands(MetadataSyncContext *context);
+extern void SendInterTableRelationshipCommands(MetadataSyncContext *context);
 
 #define DELETE_ALL_NODES "DELETE FROM pg_dist_node"
 #define DELETE_ALL_PLACEMENTS "DELETE FROM pg_dist_placement"
@@ -121,12 +182,18 @@ extern void SyncDeleteColocationGroupToNodes(uint32 colocationId);
 #define DELETE_ALL_DISTRIBUTED_OBJECTS "DELETE FROM pg_catalog.pg_dist_object"
 #define DELETE_ALL_PARTITIONS "DELETE FROM pg_dist_partition"
 #define DELETE_ALL_COLOCATION "DELETE FROM pg_catalog.pg_dist_colocation"
-#define REMOVE_ALL_SHELL_TABLES_COMMAND \
-	"SELECT worker_drop_shell_table(logicalrelid::regclass::text) FROM pg_dist_partition"
+#define DELETE_ALL_TENANT_SCHEMAS "DELETE FROM pg_catalog.pg_dist_schema"
+#define WORKER_DROP_ALL_SHELL_TABLES \
+	"CALL pg_catalog.worker_drop_all_shell_tables(%s)"
+#define CITUS_INTERNAL_MARK_NODE_NOT_SYNCED \
+	"SELECT citus_internal_mark_node_not_synced(%d, %d)"
+
 #define REMOVE_ALL_CITUS_TABLES_COMMAND \
 	"SELECT worker_drop_distributed_table(logicalrelid::regclass::text) FROM pg_dist_partition"
-#define BREAK_CITUS_TABLE_SEQUENCE_DEPENDENCY_COMMAND \
+#define BREAK_ALL_CITUS_TABLE_SEQUENCE_DEPENDENCY_COMMAND \
 	"SELECT pg_catalog.worker_drop_sequence_dependency(logicalrelid::regclass::text) FROM pg_dist_partition"
+#define BREAK_CITUS_TABLE_SEQUENCE_DEPENDENCY_COMMAND \
+	"SELECT pg_catalog.worker_drop_sequence_dependency(%s);"
 
 #define DISABLE_DDL_PROPAGATION "SET citus.enable_ddl_propagation TO 'off'"
 #define ENABLE_DDL_PROPAGATION "SET citus.enable_ddl_propagation TO 'on'"
@@ -146,6 +213,8 @@ extern void SyncDeleteColocationGroupToNodes(uint32 colocationId);
 	"placementid = EXCLUDED.placementid"
 #define METADATA_SYNC_CHANNEL "metadata_sync"
 
+#define WORKER_ADJUST_IDENTITY_COLUMN_SEQ_RANGES \
+	"SELECT pg_catalog.worker_adjust_identity_column_seq_ranges(%s)"
 
 /* controlled via GUC */
 extern char *EnableManualMetadataChangesForUser;
