@@ -330,6 +330,9 @@ typedef struct Task
 	 * Vacuum, create/drop/reindex concurrently cannot be executed in a transaction.
 	 */
 	bool cannotBeExecutedInTransction;
+
+	Const *partitionKeyValue;
+	int colocationId;
 } Task;
 
 
@@ -358,19 +361,19 @@ typedef struct JoinSequenceNode
 
 
 /*
- * InsertSelectMethod represents the method to use for INSERT INTO ... SELECT
- * queries.
+ * ModifyWithSelectMethod represents the method to use for INSERT INTO ... SELECT
+ * or MERGE type of queries.
  *
  * Note that there is a third method which is not represented here, which is
- * pushing down the INSERT INTO ... SELECT to workers. This method is executed
- * similar to other distributed queries and doesn't need a special execution
- * code, so we don't need to represent it here.
+ * pushing down the MERGE/INSERT INTO ... SELECT to workers. This method is
+ * executed similar to other distributed queries and doesn't need a special
+ * execution code, so we don't need to represent it here.
  */
-typedef enum InsertSelectMethod
+typedef enum ModifyWithSelectMethod
 {
-	INSERT_SELECT_VIA_COORDINATOR,
-	INSERT_SELECT_REPARTITION
-} InsertSelectMethod;
+	MODIFY_WITH_SELECT_VIA_COORDINATOR,
+	MODIFY_WITH_SELECT_REPARTITION
+} ModifyWithSelectMethod;
 
 
 /*
@@ -409,18 +412,22 @@ typedef struct DistributedPlan
 	Oid targetRelationId;
 
 	/*
-	 * INSERT .. SELECT via the coordinator or repartition */
-	Query *insertSelectQuery;
-	PlannedStmt *selectPlanForInsertSelect;
-	InsertSelectMethod insertSelectMethod;
+	 * Modifications performed using the output of a source query via
+	 * the coordinator or repartition.
+	 */
+	Query *modifyQueryViaCoordinatorOrRepartition;
+	PlannedStmt *selectPlanForModifyViaCoordinatorOrRepartition;
+	ModifyWithSelectMethod modifyWithSelectMethod;
 
 	/*
-	 * If intermediateResultIdPrefix is non-null, an INSERT ... SELECT
-	 * via the coordinator is written to a set of intermediate results
-	 * named according to <intermediateResultIdPrefix>_<anchorShardId>.
-	 * That way we can run a distributed INSERT ... SELECT with
-	 * RETURNING or ON CONFLICT from the intermediate results to the
-	 * target relation.
+	 * If intermediateResultIdPrefix is non-null, the source query
+	 * results are written to a set of intermediate results named
+	 * according to <intermediateResultIdPrefix>_<anchorShardId>.
+	 * That way we can run a distributed modification query which
+	 * requires evaluating source query results at the coordinator.
+	 * Once results are captured in intermediate files, modification
+	 * is done from the intermediate results into the target relation.
+	 *
 	 */
 	char *intermediateResultIdPrefix;
 
@@ -456,6 +463,13 @@ typedef struct DistributedPlan
 	 * or if prepared statement parameters prevented successful planning.
 	 */
 	DeferredErrorMessage *planningError;
+
+	/*
+	 * When performing query execution scenarios that require repartitioning
+	 * the source rows, this field stores the index of the column in the list
+	 * of source rows to be repartitioned for colocation with the target.
+	 */
+	int sourceResultRepartitionColumnIndex;
 } DistributedPlan;
 
 
@@ -540,7 +554,6 @@ extern Node *  WrapUngroupedVarsInAnyValueAggregate(Node *expression,
 													List *groupClauseList,
 													List *targetList,
 													bool checkExpressionEquality);
-extern CollateExpr * RelabelTypeToCollateExpr(RelabelType *relabelType);
 
 /*
  * Function declarations for building, updating constraints and simple operator
@@ -553,6 +566,7 @@ extern bool BinaryOpExpression(Expr *clause, Node **leftOperand, Node **rightOpe
 /* helper functions */
 extern Var * MakeInt4Column(void);
 extern int CompareShardPlacements(const void *leftElement, const void *rightElement);
+extern int CompareGroupShardPlacements(const void *leftElement, const void *rightElement);
 extern bool ShardIntervalsOverlap(ShardInterval *firstInterval,
 								  ShardInterval *secondInterval);
 extern bool ShardIntervalsOverlapWithParams(Datum firstMin, Datum firstMax,
