@@ -13,8 +13,16 @@
 
 #include "postgres.h"
 
+#include "distributed/pg_version_constants.h"
+#if PG_VERSION_NUM >= PG_VERSION_16
+#include "parser/parse_relation.h"
+#endif
 #include "utils/relcache.h"
 
 extern char * RelationGetNamespaceName(Relation relation);
+#if PG_VERSION_NUM >= PG_VERSION_16
+extern RTEPermissionInfo * GetFilledPermissionInfo(Oid relid, bool inh,
+												   AclMode requiredPerms);
+#endif
 
 #endif /* RELATION_UTILS_H */
