@@ -25,17 +25,6 @@
 #define INVALID_SHARD_ID 0
 #define INVALID_PLACEMENT_ID 0
 
-/*
- * ShardState represents last known states of shards on a given node.
- */
-typedef enum
-{
-	SHARD_STATE_INVALID_FIRST = 0,
-	SHARD_STATE_ACTIVE = 1,
-	SHARD_STATE_INACTIVE = 3,
-	SHARD_STATE_TO_DELETE = 4,
-} ShardState;
-
 
 /* Function declarations to extend names in DDL commands */
 extern void RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId);
