@@ -14,5 +14,8 @@
 #include "postgres.h"
 
 extern Oid GetTableLocalShardOid(Oid citusTableOid, uint64 shardId);
+extern char * GetLongestShardName(Oid citusTableOid, char *finalRelationName);
+extern char * GetLongestShardNameForLocalPartition(Oid parentTableOid,
+												   char *partitionRelationName);
 
 #endif /* SHARD_UTILS_H */
