@@ -15,4 +15,7 @@
 
 extern char * ConvertIntToString(int val);
 
+#define StringStartsWith(str, prefix) \
+	(strncmp(str, prefix, strlen(prefix)) == 0)
+
 #endif /* CITUS_STRING_UTILS_H */
